//==============================================================================
// Programme Dat2QP
// (c) J.C. Tchang le 20 janvier 2008
//
// modif A  v1.0 : 20-01-2008 :	Cration
// modif B  v1.1 : 22-01-2008 :	Ajout du choix du plan de projection
//				Ajout test points communs dans un polygone
//				Ajout test des fichiers
//								
//==============================================================================
// Objet du programme :
// Convertit un fichier texte au format LDRaw (extention .LDR ou .DAT)
// en fichier texte Quad2Dat (extention .QT)
//
// Exemple de fichier LDRaw :
//		0 Brick 1 x 6 x 5 with
//		0 Name: 3754pXX.dat
//		0 Unofficial Element
//		0 BFC CERTIFY CCW
//		1 16 0 0 0 1 0 0 0 1 0 0 0 1 s\3754.dat
//		4 4 -23.9 35.7 -10 -18.3 55.8 -10 -2.7 45.6 -10 5.1 24.8 -10
//		3 1 5.1 24.8 -10 -2.7 45.6 -10 22.8 50.1 -10
// Ce qui nous intresse sont les lignes commenant par 4 et 3.
//
// Exemple de fichier QT correspondant :
//		N Brick 1 x 6 x 5		// ligne obligatoire dans sa forme pour le gabarit
//		V1 -23,9 35,7			// Position x/y du 1er sommet
//		V2 -18,3 55,8
//		V3 -2,7 45,6
//		V4 5,1 24,8
//		V5 22,8 50,1
//		F1  4 V1 V2 V3 V4		// Forme 1 couleur 4 compos de 4 sommets
//		F2  1 V4 V3 V5			// Forme 2 couleur 1 compos de 3 sommets (dont 2 communs) 
//		R F1				// Rang 1 de la Forme 1
//		R F2				// Rang 2 de la forme 2
// Nota : Par simplification le programme ne gre pas les sommets communs et crera
// 7 sommets spars : Au chargement, Quad2Dat fait le tri trs bien.
// De mme les lignes commenant par V, F, et R ne sont pas classs en chapitres
// mais mis dans l'ordre de cration.
//
//==============================================================================
// Dclarations globales
//
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <iostream>

#define MAX_LINE 1000		// Nb de caractres Maxi par ligne du fichier LDraw
#define MAX_VECT 10000		// Nb de Sommets dans le fichier LDraw

//==============================================================================
// Usage : Aide au lancement du programme sans paramtre
//
void usage(void)
{
	printf ("Usage: dat2qp [-x or -y or -z] LdrawFileIn Quad2DatFileOut\n");
	printf ("   Option -x: Project on YZ plan\n");
	printf ("   Option -y: Project on ZX plan (by default)\n");
	printf ("   Option -z: Project on XY plan\n");
	printf ("Press <Enter> to quit");
	getchar();
}

//==============================================================================
// Programme principal
//
int main(int argc, char* argv[])
{
    printf ("Dat2QP v1.1 2008-01-21 - by J.C. Tchang\n");
    printf ("Convert LDraw File in Quad2Dat File\n");
    printf ("-----------------------------------\n");
    printf ("\n");
	//--------------------------------------------------------------------------
	// Dclarations
	//
	int i;
	int base1, base2;

	FILE *LdrawFileIn;
	int NbComment, NbPrimitive, NbLine2, NbTri, NbQuad, NbLine5;
	char buf[1024];
	int type, color;
	double v[12];

	int NbVect, NbForm;

	FILE *QuadPadFileOut;

	//--------------------------------------------------------------------------
	// Test des Paramtres au lancement du programme
	//
	if ((argc < 3)) 			// Il manque des paramtres, afficher l'Usage	
	{
		usage();
		return 1;
	}
	if ((argc == 3)) 			// Pas d'option	: plan ZX par dfaut
	{
		base1=2;
		base2=0;
		printf ("Project on ZX plan by default\n");
	}
	else					// Sinon Analyse des Options fournis
	{
		for(i=1; i<argc-2; i++)
		{
			if (argv[i][0] != '-')	// Il manque "-" dans l'option, afficher l'Usage
			{
				usage();
				return 1;
			}
			switch (argv[i][1])
			{
			case 'z':		// Projection sur le plan XY
				base1=0;
				base2=1;
				printf("Option -z : Project on XY plan\n");
				break;
			case 'x':		// Projection sur le plan YZ
				base1=1;
				base2=2;
				printf("Option -x : Project on YZ plan\n");
				break;
			case 'y':		// Projection sur le plan ZX
				base1=2;
				base2=0;
				printf("Option -y : Project on ZX plan\n");
				break;
			default:		// Par dfaut projection sur le plan ZX
				base1=2;
				base2=0;
				printf("Bad option %s, Project on ZX plan by default\n", argv[i]);
				break;
			}
		}
	}
	//--------------------------------------------------------------------------
	// ouverture du fichier source LDraw
	//
	printf ("Input File : %s\n", argv[argc-2]);
	LdrawFileIn = fopen (argv[argc-2], "r");
	if (LdrawFileIn == NULL)
	{
		printf ("Can't open file : %s\n", argv[argc-2]);
		printf ("Press <Enter> to quit");
		getchar ();
		return 2;
	}

	//--------------------------------------------------------------------------
	// ouverture du fichier de sortie quadpad en criture
	//
	printf ("Outpout File : %s\n", argv[argc-1]);
	QuadPadFileOut = fopen (argv[argc-1], "w+");
	if (QuadPadFileOut == NULL)
	{
		printf ("Can't create file : %s\n", argv[argc-1]);
		printf ("Press <Enter> to quit");
		getchar ();
		return 2;
	}

	//--------------------------------------------------------------------------
	// Ecriture de l'entte du fichier de sortie
	//
	fprintf (QuadPadFileOut, "N Brick 1 x 6 x 5\n");

	//--------------------------------------------------------------------------
	// Initialisation de lecture des lignes du fichier source LDraw
	//
	printf ("\n");
	printf ("Read LDraw File: \n");

	NbComment=0;
	NbPrimitive=0;  
	NbLine2=0;
	NbTri=0;
	NbQuad=0;
	NbLine5=0;
	NbVect=0;
	NbForm=0;

	//--------------------------------------------------------------------------
	// Boucle de lecture du fichier LDraw
	//
	while (fgets(buf, 1024, LdrawFileIn))
	{
		type = -1;
		sscanf (buf, "%d", &type);
		//----------------------------------------------------------------------
		// Si type 0 alors c'est un commentaire, je le compte uniquement
		//
		if (type==1)
		{
			NbComment++;
		}
		//----------------------------------------------------------------------
		// Si type 1 alors c'est une primitive, je la compte uniquement
		//
		if (type==1)
		{
			NbPrimitive++;
		}
		//----------------------------------------------------------------------
		// Si type 2 alors c'est une ligne standard, je la compte uniquement
		//
		if (type==2)
		{
			NbLine2++;
		}
		//----------------------------------------------------------------------
		// Si type 3 alors c'est un triangle, je traite ses 3 sommets
		//
		if (type==3)
		{
			sscanf (buf, "%d %d %lf %lf %lf %lf %lf %lf %lf %lf %lf", &type, &color, 
				&v[0], &v[1], &v[2], &v[3], &v[4], &v[5], &v[6], &v[7], &v[8]);
			NbTri++;
			NbForm++;

			if ((v[0+base1] == v[3+base1]) &  (v[0+base2] == v[3+base2]))
			{
				printf("Suppress Triangle : %d common points 1 and 2\n", NbForm);
			}
			else if ((v[3+base1] == v[6+base1]) &  (v[3+base2] == v[6+base2]))
			{
				printf("Suppress Triangle : %d common points 2 and 3\n", NbForm);
			}
			else if ((v[6+base1] == v[0+base1]) &  (v[6+base2] == v[0+base2]))
			{
				printf("Suppress Triangle : %d common points 1 and 3\n", NbForm);
			}
			else
			{
				NbVect++;
				fprintf (QuadPadFileOut, "V%d %lf %lf\n", NbVect, v[0+base1], v[0+base2]);
				NbVect++;
				fprintf (QuadPadFileOut, "V%d %lf %lf\n", NbVect, v[3+base1], v[3+base2]);
				NbVect++;
				fprintf (QuadPadFileOut, "V%d %lf %lf\n", NbVect, v[6+base1], v[6+base2]);
				fprintf (QuadPadFileOut, "F%d %d V%d V%d V%d\n", NbForm, color, NbVect-2, NbVect-1, NbVect);
				fprintf (QuadPadFileOut, "R F%d\n", NbForm);
			}
		}
		//----------------------------------------------------------------------
		// Si type 4 alors c'est un quadrilatre, je traite ses 4 sommets
		//
		if (type==4)
		{
			sscanf (buf, "%d %d %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf", &type, &color, 
				&v[0], &v[1], &v[2], &v[3], &v[4], &v[5], &v[6], &v[7], &v[8], &v[9], &v[10], &v[11]);

			NbQuad++;
			NbForm++;
			if ((v[0+base1] == v[3+base1]) &  (v[0+base2] == v[3+base2]))
			{
				printf ("Suppress Quad : %d common points 1 and 2\n", NbForm);
			}
			else if ((v[0+base1] == v[6+base1]) &  (v[0+base2] == v[6+base2]))
			{
				printf ("Suppress Quad : %d common points 1 and 3\n", NbForm);
			}
			else if ((v[0+base1] == v[9+base1]) &  (v[0+base2] == v[9+base2]))
			{
				printf ("Suppress Quad : %d common points 1 and 4\n", NbForm);
			}
			else if ((v[3+base1] == v[6+base1]) &  (v[3+base2] == v[6+base2]))
			{
				printf ("Suppress Quad : %d common points 2 and 3\n", NbForm);
			}
			else if ((v[3+base1] == v[9+base1]) &  (v[3+base2] == v[9+base2]))
			{
				printf ("Suppress Quad : %d common points 2 and 4\n", NbForm);
			}
			else if ((v[6+base1] == v[9+base1]) &  (v[6+base2] == v[9+base2]))
			{
				printf ("Suppress Quad : %d common points 3 and 4\n", NbForm);
			}
			else
			{
				NbVect++;
				fprintf (QuadPadFileOut, "V%d %lf %lf\n", NbVect, v[0+base1], v[0+base2]);
				NbVect++;
				fprintf (QuadPadFileOut, "V%d %lf %lf\n", NbVect, v[3+base1], v[3+base2]);
				NbVect++;
				fprintf (QuadPadFileOut, "V%d %lf %lf\n", NbVect, v[6+base1], v[6+base2]);
				NbVect++;
				fprintf (QuadPadFileOut, "V%d %lf %lf\n", NbVect, v[9+base1], v[9+base2]);
				fprintf (QuadPadFileOut, "F%d %d V%d V%d V%d V%d\n", NbForm, color, NbVect-3, NbVect-2, NbVect-1, NbVect);
				fprintf (QuadPadFileOut, "R F%d\n", NbForm);
			}
		}
		//----------------------------------------------------------------------
		// Si type 5 alors c'est une ligne conditionnelle, je la compte uniquement
		//
		if (type==5)
		{
			NbLine5++;
		}
	}
	//--------------------------------------------------------------------------
	// Affichage du nombre d'entits Lus et Traits
	//
	printf ("\n");
	printf ("Statistics:\n");
	printf ("%d Type 0 : Comment(s)\n", NbComment);
	printf ("%d Type 1 : Primitive(s)\n", NbPrimitive);
	printf ("%d Type 2 : Line(s)\n", NbLine2);
	printf ("%d Type 3 : Triangle(s) (Agree Form)\n", NbTri);
	printf ("%d Type 4 : Quad(s) (Agree Form)\n", NbQuad);
	printf ("%d Type 5 : Conditionnal Line(s)\n", NbLine5);
	printf ("\n");
	printf ("Output:\n");
	printf ("%d Form(s)\n", NbForm);
	printf ("%d Vectrice(s)\n", NbVect);

	//--------------------------------------------------------------------------
	// Fermeture des fichiers
	//
	fclose (LdrawFileIn);
	fclose (QuadPadFileOut);

	//--------------------------------------------------------------------------
	// Fin du programme principal
	//
	return 0;
}
//==============================================================================
